#This script file runs the Replica Manager Election Protocol

Class Density 


# =======================================================================
# Settings
#Communication parameters

set opt(chan)           Channel/WirelessChannel
set opt(prop)           Propagation/TwoRayGround
set opt(netif)          Phy/WirelessPhy
set opt(mac)            Mac/802_11
set opt(ifq)            Queue/DropTail/PriQueue
set opt(ll)             LL
set opt(ant)            Antenna/OmniAntenna
 
set opt(x)              [lindex $argv 2]           ;# X dimension of the topography
set opt(y)              [lindex $argv 2]           ;# Y dimension of the topography
set opt(grid)           100     ;# grid size
set opt(cp)             "../mobility/scene/cbr-10-1-10-20"
set opt(sc)             [lindex $argv 0]           ;#[lindex $argv 0]

 
set opt(ifqlen)         50                         ;# max packet in ifq
set opt(nn)             [lindex $argv 1]           ;# number of nodes
set opt(seed)           0.0
set opt(stop)           1201.0          ;# simulation time
set opt(tr)             tempElection.rands          ;# trace file
set opt(rp)             AODV        ;# routing protocol script: DumbAgent
set opt(lm)             "off"           ;# log movement
set opt(energymodel)    EnergyModel     ;
set opt(initialenergy)  500             ;# Initial energy in Joules
set opt(infiniteenergy)  20000           ;# enough to keep sim run to the end


# =====================================================================
# Other default settings

LL set mindelay_		50us
LL set delay_			25us
LL set bandwidth_		0	;# not used


Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
Antenna/OmniAntenna set X_ 0
Antenna/OmniAntenna set Y_ 0
Antenna/OmniAntenna set Z_ 1.5
Antenna/OmniAntenna set Gt_ 1.0
Antenna/OmniAntenna set Gr_ 1.0

# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface
Phy/WirelessPhy set CPThresh_ 10.0
Phy/WirelessPhy set CSThresh_ 1.559e-11
Phy/WirelessPhy set RXThresh_ 3.652e-10
Phy/WirelessPhy set Rb_ 2*1e6
Phy/WirelessPhy set Pt_ 0.28183815
Phy/WirelessPhy set freq_ 914e+6 
Phy/WirelessPhy set L_ 1.0

# =====================================================================


# ==========TRACING===================
set AgentTrace ON
set RouteTrace ON



# ==========GENERAL PROCEDURES=========



proc usage {} {
	global argv0
	puts stderr "usage: ns esElection.tcl <topology_filename> <participants_number> <square_dimensions> <initiator_node>"
	puts "Valid Tests: dsdv dsr"
	exit 1
}




proc finish {} {
	$self instvar ns_
	global quiet opt nf

	$ns_ flush-trace
        
        set tracefd	[open $opt(tr) r]
        
	close $tracefd
	close $nf
	
	puts "finish.."
	exit 0
	
#====================================================================
#======================SCENARIO SETTINGS=============================
#====================================================================

        set ns_         [new Simulator]	
	set topo	[new Topography]
	set tracefd	[open $opt(tr) w]
	
	$ns_ trace-all $tracefd
	set nf [open out2.nam w]
        $ns_ namtrace-all-wireless $nf  $opt(x) $opt(y)
	set curr ""

	
	$topo load_flatgrid $opt(x) $opt(y)

	
	puts $tracefd "M 0.0 nn:$opt(nn) x:$opt(x) y:$opt(y) rp:$opt(rp)"
	puts $tracefd "M 0.0 sc:$opt(sc) cp:$opt(cp) seed:$opt(seed)"
	puts $tracefd "M 0.0 prop:$opt(prop) ant:$opt(ant)"

	#
	# Create God
	#
	set god_ [create-god $opt(nn)]
	
        	
        #==========================================================
        #==================NODES===================================
        #==========================================================

	#General nodes configuration
        $ns_ node-config -adhocRouting $opt(rp) \
                         -llType $opt(ll) \
                         -macType $opt(mac) \
                         -ifqType $opt(ifq) \
                         -ifqLen $opt(ifqlen) \
                         -antType $opt(ant) \
                         -propType $opt(prop) \
                         -phyType $opt(netif) \
                         -channelType $opt(chan) \
                         -topoInstance $topo \
                         -energyModel $opt(energymodel) \
                         -routerTrace OFF \
                         -macTrace OFF \
                         -rxPower 1.2 \
                         -txPower 1.6 \
                         -idlePower 1.15 \
                         -initialEnergy $opt(infiniteenergy) 

  
 	for {set i 0} {$i < $opt(nn) } {incr i} {
                
		set node_($i) [$ns_ node]
                $node_($i) random-motion 0              ;# disable random motion
		$node_($i) attach-densitypartner
                
		$ns_ initial_node_pos $node_($i) 250
		$node_($i) color red
		$ns_ at 0.01 "$node_($i) color red"
        }                   
        
	#==========================================================
        #==================AGENTS==================================
        #==========================================================
        for {set i 0} {$i < $opt(nn)} {incr i} {
	   	
	        set den_($i) [new Agent/Density [$node_($i) id]]
		$node_($i) attach $den_($i) 300
		
		}
	
	Agent/Density disc_delay 5
        Agent/Density part_adv_interval 100
	Agent/Density part_adv_delay 5
	Agent/Density limit 11
	Agent/Density dist_req_delay 1.5
        Agent/Density dist_rep_interval 6
	#When I can stop election 
	Agent/Density election_accuracy 0.5
	#How much time I wait to delegate the Election to another node 
	#deve essere tarato considerando il dist_rep_interval
	Agent/Density delegation_interval 100
	#What is the probability a node forwards the distance calculation request 
	Agent/Density forwarding_prob 200
	#The probability that a distant node replies 
	Agent/Density dist_reply_prob 200
	#The max number of best solutions explored in a raw
	Agent/Density max_best 3
        

#============================================================================
# Script procedures invoked within the density.cc code
#============================================================================
        
Agent/Density instproc recv { id2 from} {
        $self instvar node_
        #puts "node [$node_ id] received density request from \
        #      $from (and I am $id2)"
}

Agent/Density instproc vis {num } {
        
	$self instvar node_ 
	
         #puts "node [$node_ id] has $num neighbors "
}

Agent/Density instproc neigh { from num} {
        $self instvar node_	
	puts "node [$node_ id] knows that  \
              $from has $num neighbors"
}

Agent/Density instproc dump { node num} {
        global ns_
	$self instvar node_
        puts "[$node_ id]:$node has $num neighbors"
}

Agent/Density instproc initiator {} {
       $self instvar node_
        puts "node [$node_ id] is the initiator"
}

Agent/Density instproc mytable {neigh num} {
       $self instvar node_
       global curr
       puts "node [$node_ id]: in my table there is $neigh with $num neighbors"
       
}

Agent/Density instproc sendto {neigh id} {
       $self instvar node_
        puts "node [$node_ id]: I would send my first packet to $neigh"
}

Agent/Density instproc chain_Recv { from num} {
        $self instvar node_
        puts "node [$node_ id] received a chain message from $from"
}

Agent/Density instproc backto { to num} {
        $self instvar node_
        puts "node [$node_ id] send in backward chaining to $to"
}

Agent/Density instproc backtoRecv { from num} {
        $self instvar node_
        puts "node [$node_ id] receive in backward chaining from $from"
}

Agent/Density instproc tab { id } {
        global node_ ns_ opt
	set myvar $opt(stop)-10
	puts "chain member: $id  "
	$ns_ at 397 "$node_($id) color blue"
}
Agent/Density instproc out { limit next } {
        puts "limite: $limit  and next entry is $next"	
}

Agent/Density instproc front {node} {
       $self instvar node_
       puts "[$node_ id] fronteer is $node"
}

Agent/Density instproc expl {} {
       global ns_
       $self instvar node_
       set id [$node_ id]
       puts "[$node_ id] has been explored"
       global node_
       $ns_ at 1110 "$node_($id) color yellow"
}

Agent/Density instproc end {val} {
       global ns_ 
       $self instvar node_
       set id [$node_ id]
       puts "[$node_ id] value is $val"
       global node_
       $ns_ at 1151 "$node_($id) color white"
}

Agent/Density instproc reply { node } {
        global ns_ node_
	puts "reply from $node"	
	#$ns_ at 498 "$node_($node) color green"
	
}

set t  0 


Agent/Density instproc ipart2 { res time} {
       global ns_ t 
       $self instvar node_
       set id [$node_ id]
       puts "$time: $id participate ?  $res"
       global node_
       set cur [$ns_ now]
       set cur2 [expr ($cur + 0.001)]	
       if {$res==1} {$ns_ at $cur2 "$node_($id) color blue" 
		     set t [ expr {$t + 1}]}
       if {$res==0} {$ns_ at $cur2 "$node_($id) color black"
		     set t [ expr {$t - 1}]}
       puts "$id Numero di appartenenti A $time: $t"

       } 

Agent/Density instproc dumpN {n} {
       $self instvar node_
       set id [$node_ id]
       puts "Neighbor table of $id: $n"
       }

Agent/Density instproc ipart { res } {
       global ns_ t 
       $self instvar node_
       set id [$node_ id]
       set out "$id Participate ?  $res"
       global node_
       set cur [$ns_ now]
       set cur2 [expr ($cur + 0.001)]	
       if {$res==1} {$ns_ at 300 "$node_($id) color blue" 
		     set t [ expr {$t + 1}]}
       if {$res==0} {$ns_ at 300 "$node_($id) color black"
		     }; #set t [ expr {$t + 1}]
                    
       append out " $id Numero di partecipanti al protocollo: $t"
       puts $out
       } 

Agent/Density instproc ping {val} {
       $self instvar node_
       set id [$node_ id]
       puts "                                                  $id received ping from $val"
}

proc out {num idp } {
puts "Distance: the node is $num, the distance is $idp"}

proc out3 {num idp id2} {
puts "Reply: node $num, forwarder $idp, source $id2"}

proc out4 {num idp id2} {
puts "Request: node $num, forwarder $idp, source $id2"}

proc out6 {num idp } {
puts "Forwarding: node $num for node $idp"}

proc interrupt {id1 id2} {
puts "Interrupt: node $id1 interrupted by node $id2"}

proc reply {me next id} {
puts "Node $me replies to $next towards $id"}

proc outHello {me ne} {
puts "Node $me: one of my neighbor is $ne"}

proc outDelHello {me ne} {
puts "Node $me: I can't see $ne" }

proc outSendHello {me} {
puts "I send my Hello $me"}

proc outSolicit {me time} {
puts "$me solicits at $time"} 

set countedButShouldNot 0
set notCountedButShould 0

proc test1 {me time} {
puts "TEST1 $me $time"}

proc testLauncher {} {
global opt den_ ns_
for {set i 0} {$i < $opt(nn) } {incr i} {
     set cur [$ns_ now]
     set timer [expr {$cur + 1}]
     $ns_ at $timer "$den_($i) count"}
}
proc totale {} {
       global t
       puts "PARTECIPANTI TOTALI: $t"
      } 

#============================================================================
#                        END OF SCRIPT PROCEDURES
#============================================================================





#============================================================================
#                         SCENARIO EVENTS
#============================================================================

#NS2 START
puts "Loading scenario file..."
source $opt(sc)
puts "Load complete..."

#STOP
for {set i 0} {$i < $opt(nn) } {incr i} {
        $ns_ at $opt(stop).000000001 "$node_($i) reset";
}
$ns_ at $opt(stop).00000001 "puts \"NS EXITING...\" ; $ns_ halt"
	
#SCENARIO START
$ns_ at 5.0 "$den_(0) start"
$ns_ at 150 "totale"

#Choice of the election initiator node
set x [lindex $argv 3]
if {$argc=="3"} {set x 0}
$ns_ at 150 "$den_($x) calcDist"

#RUN
$ns_ run







                        
